#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <stdlib.h>
#include <error.h>
#include <errno.h>
#include <unistd.h>
#include <pwd.h>
#include <grp.h>

#include <sys/capability.h>
#include <sys/prctl.h>

#include "droppriv.h"

uid_t user_uid = -1;

void
priv_minimize(void)
{
	const char *dir = "/var/resolv";
	cap_t   caps;
	uid_t   uid;
	gid_t   gid;

	user_uid = getuid();
	if (user_uid && geteuid())
		return;

	if (setgroups(0, NULL) < 0)
		error(EXIT_FAILURE, errno, "setgroups");

	if (prctl(PR_SET_KEEPCAPS, 1))
		error(EXIT_FAILURE, errno, "prctl PR_SET_KEEPCAPS failed");

	if (user_uid == 0)
	{
		const char user[] = "iputils";
		struct passwd *pw = getpwnam(user);

		if (!pw)
			error(EXIT_FAILURE, 0, "lookup of user \"%s\" failed",
			      user);

		gid = pw->pw_gid;
		uid = pw->pw_uid;
		endpwent();

		if (uid == 0)
			error(EXIT_FAILURE, 0,
			      "user \"%s\" shouldn't be root", user);
	} else
	{
		uid = user_uid;
		gid = getgid();
	}

	if (chroot(dir) < 0 || chdir("/") < 0)
		error(EXIT_FAILURE, errno, "chroot to \"%s\" failed", dir);

	if (setgid(gid) < 0)
		error(EXIT_FAILURE, errno, "setgid");

	caps = cap_from_text
		("cap_setuid,cap_net_raw,cap_net_bind_service=ep");
	if (!caps)
		error(EXIT_FAILURE, errno, "cap_from_text failed");

	if (cap_set_proc(caps) < 0)
		error(EXIT_FAILURE, errno, "cap_set_proc failed");

	cap_free(caps);

	if (setreuid(uid, uid) < 0)
		error(EXIT_FAILURE, errno, "setreuid");

	caps = cap_from_text("cap_net_raw,cap_net_bind_service=ep");
	if (!caps)
		error(EXIT_FAILURE, errno, "cap_from_text failed");

	if (cap_set_proc(caps) < 0)
		error(EXIT_FAILURE, errno, "cap_set_proc failed");

	cap_free(caps);
}

void
priv_drop(void)
{
	cap_t   caps;

	caps = cap_from_text("all-ep");
	if (!caps)
		error(EXIT_FAILURE, errno, "cap_from_text failed");

	if (cap_set_proc(caps) < 0)
		error(EXIT_FAILURE, errno, "cap_set_proc failed");

	cap_free(caps);
}
