/* -*- Mode: c; tab-width: 4; indent-tabs-mode: 1; c-basic-offset: 4; -*- */
/* vim:set ts=4 sw=4 ai nobackup nocindent sm: */

/*
 * tcptraceroute -- A traceroute implementation using TCP packets
 * Copyright (c) 2001-2006  Michael C. Toren <mct@toren.net>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * A copy of the GNU GPL is available as /usr/doc/copyright/GPL on Debian
 * systems, or on the World Wide Web at http://www.gnu.org/copyleft/gpl.html
 * You can also obtain it by writing to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#if defined(__GNUC__) && defined(__GNUC_MINOR__)
# define TCPTRACEROUTE_GNUC_PREREQ(maj, min) \
        ((__GNUC__ << 16) + __GNUC_MINOR__ >= ((maj) << 16) + (min))
#else
# define TCPTRACEROUTE_GNUC_PREREQ(maj, min) 0
#endif

#if TCPTRACEROUTE_GNUC_PREREQ(2,5)
# define TCPTRACEROUTE_FORMAT(params) __attribute__((__format__ params))
# define TCPTRACEROUTE_NORETURN __attribute__((__noreturn__))
#else
# define TCPTRACEROUTE_FORMAT(params)
# define TCPTRACEROUTE_NORETURN
#endif

#define IPTOSBUFFERS	12
#define IPTOSBUFSIZ		(4*3+3+1)  /* Four three-digit numbers, three dots, and NUL */

/*
 * A generic wrapper for libnet_name_resolve and libnet_name2addr4, because
 * it's annoying to have #ifdef's all over the place to support both versions
 * of libnet.
 */

#if (LIBNET_API_VERSION < 110)
#define hosttoip(hostname, numeric) \
	libnet_name_resolve((u_char *)hostname, numeric)
#else
#define hosttoip(hostname, numeric) \
	libnet_name2addr4(libnet_context, (char *)hostname, numeric)
#endif

void fatal(char *, ...) TCPTRACEROUTE_NORETURN TCPTRACEROUTE_FORMAT((__printf__, 1, 2));
void debug(char *, ...) TCPTRACEROUTE_FORMAT((__printf__, 1, 2));
void warn(char *, ...) TCPTRACEROUTE_FORMAT((__printf__, 1, 2));
void pfatal(char *) TCPTRACEROUTE_NORETURN;
void print_help(void) TCPTRACEROUTE_NORETURN;
void print_usage(char *, ...) TCPTRACEROUTE_NORETURN TCPTRACEROUTE_FORMAT((__printf__, 1, 2));
void about(void) TCPTRACEROUTE_NORETURN;
void *xrealloc(void *, int);
char *safe_strncpy(char *, char *, int);
int safe_snprintf(char *, int, char *, ...) TCPTRACEROUTE_FORMAT((__printf__, 3, 4));
char *sprintable(char *);
int isnumeric(char *);
struct timeval tvdiff(struct timeval *, struct timeval *);
int tvsign(struct timeval *);
char *iptos(u_long);
char *iptohost(u_long);
void debugoptions(void);
